open Circulationfile
open Gfile
open Tools
open Ffalgo


let flow_to_string gr = gmap gr (fun fl -> String.concat "/" [(string_of_int fl.current); (string_of_int fl.capacity)])

let () =

  (* Check the number of command-line arguments *)
  if Array.length Sys.argv <> 3 then
    begin
      Printf.printf "\nUsage: %s infile outfile\n\n%!" Sys.argv.(0) ;
      exit 0
    end ;


  (* Arguments are : infile(1) source-id(2) sink-id(3) outfile(4) *)

  let infile = Sys.argv.(1)
  and outfile = Sys.argv.(2)
  in

  (* Open file *)
  let initial_network = circulation_from_file infile in
  (* Execute FF *)
  let solution_network = run_ff initial_network in
  (* Print max flow *)
  let () = Printf.printf "Max flow: %d\n%!" (get_max_flow solution_network) in
  (* Convert to string for export *)
  let final_graph = flow_to_string solution_network.graph in

  (* Rewrite the graph that has been read. *)
  let () = export outfile final_graph in

  ()

