open Gfile
open Tools
open Ffalgo


let flow_to_string gr = gmap gr (fun fl -> String.concat "/" [(string_of_int fl.current); (string_of_int fl.capacity)])

let test_ffalgo_init gr =
  let graph_init = initialize_graph gr in
  let () = export "graph_init" (flow_to_string graph_init) in
  graph_init

let test_ffalgo_apply gr =
  let graph_apply = apply_path gr [(0, 3); (3, 1); (1, 4); (4, 5)] 1 in
  let () = export "graph_apply" (flow_to_string graph_apply) in
  graph_apply

let test_ffalgo_res gr =
  let graph_res = build_res_graph gr in
  let () = export "graph_res" (gmap graph_res string_of_int) in
  graph_res

let test_ffalgo_min gr =
  let min = find_min gr [(0, 3); (3, 1); (1, 4); (4, 5)] in
  Printf.printf "min: %d\n%!" min

let test_ffalgo_path gr =   
  let predecessors = find_path {
      graph = gr;
      origin = 0;
      destination = 4;
    } in
  List.iter (fun (o, d) -> Printf.printf "%d -> %d\n%!" o d ) predecessors 

let test_ffalgo gr = 
  let graph_init = test_ffalgo_init gr in
  let graph_apply = test_ffalgo_apply graph_init in
  let graph_res = test_ffalgo_res graph_apply in
  let () = test_ffalgo_min gr in
  test_ffalgo_path graph_res


let () =

  (* Check the number of command-line arguments *)
  if Array.length Sys.argv <> 5 then
    begin
      Printf.printf "\nUsage: %s infile source sink outfile\n\n%!" Sys.argv.(0) ;
      exit 0
    end ;


  (* Arguments are : infile(1) source-id(2) sink-id(3) outfile(4) *)

  let infile = Sys.argv.(1)
  and outfile = Sys.argv.(4)
  and source = int_of_string Sys.argv.(2)
  and sink = int_of_string Sys.argv.(3)
  in

  (* Open file *)
  let graph = from_file infile in
  (* Convert graph *)
  let int_graph = (gmap graph int_of_string) in
  (* Execute FF *)
  let flow_network = run_ff {graph = int_graph; origin = source; destination = sink} in
  (* Print max flow *)
  let () = Printf.printf "Max flow: %d\n%!" (get_max_flow flow_network) in
  (* Convert to string for export *)
  let final_graph = flow_to_string flow_network.graph in

  (* TESTS *)
  (* let () = test_ffalgo (gmap graph int_of_string) in *)
  (* let graph2 = clone_nodes graph in *)
  (* let graph2 = gmap (gmap (gmap graph int_of_string) (fun x -> x + 1)) string_of_int in *)
  (* let graph2 = gmap (add_arc (gmap graph int_of_string) 2 1 13) string_of_int in *)
  (* END TESTS *)

  (* Rewrite the graph that has been read. *)
  let () = export outfile final_graph in

  ()

